// 必要なパッケージのインポート
const St = imports.gi.St;
const Main = imports.ui.main;
const Lang = imports.lang;
const GLib = imports.gi.GLib;
const Gio = imports.gi.Gio;
const PanelMenu = imports.ui.panelMenu;
const PopupMenu = imports.ui.popupMenu;
const Mainloop = imports.mainloop;


// DropboxIndicatorクラスの定義
const DropboxIndicator = new Lang.Class({
  Name: 'DropboxIndicator',

  // PanelMenu.Buttonクラスを継承 (必須)
  Extends: PanelMenu.Button,

  // コンストラクタ
  _init: function() {
    // 基底クラスのコンストラクタの呼び出し
    this.parent(0.0, "Dropbox Indicator");

    // アイコンオブジェクトの定義
    // icon_name: アイコンの種類を指定
    //   アイコン一覧は/usr/share/iconsなどを参照
    // style_class: 適用するCSSスタイル
    //   stylesheet.cssなどで独自にスタイルを定義することが可能
    let icon = new St.Icon({ icon_name: 'dropbox',
                             style_class: 'system-status-icon' });

    // パネルメニューに登録
    this.actor.add_actor(icon);

    // レイアウトの定義
    // ここではBoxLayoutを採用
    this._box = new St.BoxLayout({ style_class: 'panel-status-button' });

    // レイアウトを適用
    this.actor.add_actor(this._box);

    // 'Open Folder'ボタンの定義
    let open_folder_btn = new PopupMenu.PopupMenuItem("Open folder");

    // ボタンの登録
    this.menu.addMenuItem(open_folder_btn);

    // 'Open Folderボタンの定義'
    let turn_off_btn = new PopupMenu.PopupMenuItem("Turn off");
    this.menu.addMenuItem(turn_off_btn);

    // クリックハンドラの設定
    // Lang.bindで無名関数内のthisの値を指定できる。
    open_folder_btn.connect('activate', Lang.bind(this, function(){
      this.openFolder();
    }));

    turn_off_btn.connect('activate', Lang.bind(this, function(){
      this.turnOff();
    }));
  },

  turnOff: function() {
    // `dropbox stop`というコマンドを非同期で実行
    let [ok, pid] = GLib.spawn_async(null, ["dropbox", "stop"], null, GLib.SpawnFlags.SEARCH_PATH, null);
  },

  openFolder: function() {
    let ok, content, pid;

    // パスの連結
    // Pythonでいうos.path.join関数に相当
    let dropbox_config_path = GLib.build_filenamev([GLib.get_home_dir(), ".dropbox", "info.json"]);

    // Dropboxの設定ファイルの中身を取得
    [ok, content] = GLib.file_get_contents(dropbox_config_path);

    // 同期しているローカルフォルダのパスを取得
    let sync_dir = JSON.parse(content)["personal"]["path"];

    // ログに出力
    log("sync_dir: " + sync_dir);

    // `xdg-openコマンドでディレクトリを開く`
    [ok, stdout, stderr, exit_status] = GLib.spawn_sync(null, ["xdg-open", sync_dir], null, GLib.SpawnFlags.SEARCH_PATH, null);
    if (!ok) {
      log("Failed to open directory");
    }
  }
});

let dbmenu = null;

function set_timer() {
  // タイマーオブジェクトの生成
  // JavaScriptのsetTimeout関数に相当
  // 2秒後に再度同期状態を取得する
  Mainloop.timeout_add_seconds(2, function(){
    update();
  });
}

function update() {
  set_timer();

  // `dropbox start`コマンドの実行
  let [ok, stdout, stderr, exit_status] = GLib.spawn_sync(null, ["dropbox", "running"], null, GLib.SpawnFlags.SEARCH_PATH, null);

  let status_str;
  if (dbmenu === null && exit_status > 0) {
    // dropboxが稼働している状態
    dbmenu = new DropboxIndicator;

    // パネルメニューをトップバーに表示
    Main.panel.addToStatusArea('dropbox-indicator', dbmenu, 0, "right");
  } else if (dbmenu !== null && exit_status == 0) {
    // dropboxが稼働していない状態

    // パネルメニューの破棄
    dbmenu.destroy();
    dbmenu = null;
  }
}

// 拡張機能のインストール後、およびgnome-shellプロセスの生成後に実行
function init() {}

// 拡張機能を有効にした時に実行
function enable() {
  update();
}

// 拡張機能が無効になった際に実行
function disable() {
  if (dbmenu !== null){
    dbmenu.destroy();
  }
}
